/*
 * Created on 23.09.2008
 *
 */
package com.eaio.twitterbackup.ui;

import java.awt.Toolkit;
import java.io.IOException;
import java.util.Enumeration;
import java.util.prefs.BackingStoreException;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.text.StrBuilder;
import org.apache.log4j.Appender;
import org.apache.log4j.AppenderSkeleton;
import org.apache.log4j.Logger;
import org.apache.log4j.spi.LoggingEvent;

import thinlet.FrameLauncher;
import thinlet.Thinlet;

import com.eaio.twitterbackup.TwitterBackup;
import com.eaio.twitterbackup.TwitterBackupController;

/**
 * The TwitterBackup UI. Uses Thinlet so most of the GUI stuff is in ui.xml.
 * 
 * @author <a href="mailto:jb&#64;eaio.com">Johann Burkard</a>
 * @version $Id: TwitterBackupUI.java 6465 2014-07-04 22:00:32Z johann $
 */
public class TwitterBackupUI extends Thinlet {

    private static final long serialVersionUID = -8310719070327363473L;

    private static final Logger LOG = Logger.getLogger(TwitterBackupUI.class);

    private static final String ENABLED = "enabled";

    private static final String TEXT = "text";

    private TwitterBackupController controller;

    private Thread controllerThread;

    private StrBuilder logBuilder = new StrBuilder();

    /**
     * Constructor for TwitterBackupUI.
     *
     * @throws IOException
     */
    @SuppressWarnings("unchecked")
    public TwitterBackupUI() throws IOException {
        super();
        add(parse("/com/eaio/twitterbackup/ui/ui.xml"));

        /* Logging setup */

        Appender a = new LogPanelAppender();

        Appender app = null;
        for (Enumeration<Appender> e = Logger.getRootLogger().getAllAppenders(); e.hasMoreElements();) {
            if ((app = e.nextElement()).getLayout() != null) {
                a.setLayout(app.getLayout());
            }
        }
        Logger.getRootLogger().removeAllAppenders();
        Logger.getRootLogger().addAppender(a);

        LOG.info(TwitterBackup.VERSION + " by Johann Burkard. http://is.gd/4ete");

        /* Controller setup */

        controller = new TwitterBackupController();
        controller.restore();
        setString(find("username"), TEXT, controller.getUsername());
        setString(find("password"), TEXT, controller.getPassword());
        setString(find("file"), TEXT, controller.getFile());
        if (controller.getProxy() != null) {
            setString(find("proxy"), TEXT, controller.getProxy()
                    + (controller.getPort() != 80 ? ":" + (int) controller.getPort() : ""));
        }
        setBoolean(find("start"), ENABLED, controller.getUsername() != null && controller.getPassword() != null
                && controller.getFile() != null);
    }

    public void setUsername(String username) {
        controller.setUsername(username);
        setBoolean(find("start"), ENABLED, controller.getUsername() != null && controller.getPassword() != null
                && controller.getFile() != null);
    }

    public void setPassword(String password) {
        controller.setPassword(password);
        setBoolean(find("start"), ENABLED, controller.getUsername() != null && controller.getPassword() != null
                && controller.getFile() != null);
    }

    public void setFile(String file) {
        controller.setFile(file);
        setBoolean(find("start"), ENABLED, controller.getUsername() != null && controller.getPassword() != null
                && controller.getFile() != null);
    }

    public void setProxy(String proxy) {
        String p = StringUtils.trimToNull(proxy);
        int cIdx = proxy.indexOf(':');
        if (p == null) {
            controller.setProxy(null);
            controller.setPort((char) 0);
        }
        else if (cIdx != -1 && cIdx != p.length() - 1) {
            String host = proxy.substring(0, proxy.indexOf(':'));
            char port = (char) Integer.parseInt(proxy.substring(proxy.indexOf(':') + 1));
            controller.setProxy(host);
            controller.setPort(port);
        }
        else {
            controller.setProxy(p);
            controller.setPort((char) 80);
        }
    }

    public void action() {
        if (controllerThread == null) {

            setBoolean(find("username"), ENABLED, false);
            setBoolean(find("password"), ENABLED, false);
            setBoolean(find("file"), ENABLED, false);
            setBoolean(find("proxy"), ENABLED, false);
            setString(find("start"), TEXT, "Stop");

            controllerThread = new Thread(controller);
            controllerThread.setDaemon(true);
            controllerThread.start();

            Thread t = new Thread(new Runnable() {

                /**
                 * @see java.lang.Runnable#run()
                 */
                public void run() {
                    try {
                        controllerThread.join();
                    }
                    catch (InterruptedException ex) {
                        // Ignored.
                    }
                    controllerThread = null;
                    setBoolean(find("username"), ENABLED, true);
                    setBoolean(find("password"), ENABLED, true);
                    setBoolean(find("file"), ENABLED, true);
                    setBoolean(find("proxy"), ENABLED, true);
                    setString(find("start"), TEXT, "Start");
                    setBoolean(find("start"), ENABLED, true);
                }

            });
            t.setDaemon(true);
            t.start();
        }
        else {
            setBoolean(find("start"), ENABLED, false);
            controllerThread.interrupt();
        }
    }

    private synchronized void appendToLog(String s) {
        Object log = find("log");
        logBuilder.append(s);
        setString(log, TEXT, logBuilder.toString());
    }

    public static void main(String[] args) throws IOException {
        final TwitterBackupUI ui = new TwitterBackupUI();
        FrameLauncher launcher = new FrameLauncher(TwitterBackup.VERSION, ui, 600, 338);
        launcher.setIconImage(Toolkit.getDefaultToolkit().createImage(TwitterBackupUI.class.getResource("favicon.png")));
        Runtime.getRuntime().addShutdownHook(new Thread() {

            public void run() {
                try {
                    ui.controller.save();
                }
                catch (BackingStoreException ex) {
                    // Ignored
                }
            }

        });
    }

    class LogPanelAppender extends AppenderSkeleton {

        /**
         * Constructor for LogPanelAppender.
         */
        public LogPanelAppender() {
            super();
        }

        /**
         * @see org.apache.log4j.AppenderSkeleton#append(org.apache.log4j.spi.LoggingEvent)
         */
        protected void append(LoggingEvent event) {
            appendToLog(layout.format(event));
        }

        /**
         * @see org.apache.log4j.Appender#close()
         */
        public void close() {
        // Blank
        }

        /**
         * @see org.apache.log4j.Appender#requiresLayout()
         */
        public boolean requiresLayout() {
            return true;
        }

    }

}
